//packetAnalysisDemo_Part1 by Andrew Rose
//Experiment: Measure Data Transfer Rate (speed)

//Part 1: Statistics
//1.1: sum bytes
//1.2: calculate statistics
//  a) elapsed time
//  b) packets/sec
//  c) avg packet size
//  d) data transfer rate
//1.3: add a function to convert to appropriate units
//1.4: output text as html using a div
//Challenge: separate layer 2 and 3 packets

//1.4: global div variable 
var statsDiv;
//global variables needed for print/display functions
var total, time, bytes;

function setup() {
  createFileInput(gotFile); //user must select valid wireshark JSON
  statsDiv = createDiv("") //1.4
}

//called whenever a file is selected
function gotFile(file) {
  loadJSON(file.data, parseJSON);
}

//called once JSON is loaded
function parseJSON(packets) {
  total = packets.length
  //create a variable to count bytes
  bytes = 0
    //look at each packet i 
  for (var i = 0; i < packets.length; i++) {
    //create a layer variable
    var layer = packets[i]._source.layers; //['_source']['layers'];

    //1.1: sum bytes
    bytes += parseInt(layer.frame["frame.len"]);
    //print individual lengths to verify they add up to total
    console.log("Packet " + i + ": " + layer.frame["frame.len"])

    //calculate time duration
    if (i == packets.length - 1) //last packet
      time = layer.frame["frame.time_relative"]
  }
  printStats() //1.2-3
  displayStats() //1.4
  
}
//1.2: calculate statistics
function printStats() {
  console.log("---STATISTICS---")
  console.log("Total Length: " + bytes + "B");
  console.log("Total Length: " + convert(bytes,"B")); //1.3
  console.log("Packets: " + total);
  //nf: number format - see documentation for use
  console.log("Avg Packet Size " + nf(bytes / total, -1, 2) + "B");
  console.log("Elapsed Time: " + nf(time, -1, 2) + "s");
  console.log("Packets/sec: " + nf((total / time), -1, 2));
  console.log("Data Rate: " + nf(bytes * 8 / time,-1,2)+"bps");
  console.log("Data Rate: " + convert(bytes * 8 / time,"bps")); //1.3
}

//1.3: convert to the appropriate unit ('B' or 'bps')
function convert(data,unit) {
  var i = 0;
  while (data >= 1000) {
    data /= 1000;
    i++;
  }
  data = nf(data, -1, 2);
  
  if (i == 0)
    return data + " " + unit
  else if (i == 1) //Kilo
    return data + " K" + unit
  else if (i == 2) //Mega
    return data + " M" + unit
  else //Giga
    return data + " G" + unit
}

//1.4: display statistics as html
function displayStats() {
  var stats = "";
  stats += "Elapsed Time: " + nf(time, -1, 2) + "s <br>";
  stats += "Packets: " + total + "<br>";
  stats += "Packets/sec: " + nf((total / time), -1, 2) + "<br>";
  stats += "Data: " + convert(bytes,"B") + "<br>";
  stats += "Avg Packet Size " + convert(bytes/total, "B") + "<br>";
  stats += "Data Rate: " + convert(bytes*8/time, "bps") + "<br>";
  statsDiv.html(stats)
}